@@ -22,7 +22,6 @@
 #include <QQueue>
 #include <QElapsedTimer>
 #include <QTimer>
-#include <QDebug>
 
 #include "syncfileitem.h"
 
@@ -42,6 +41,35 @@ public:
      */
     void reset();
 
+    /** Records the status of the sync run
+     */
+    enum Status {
+        /// Emitted once at start
+        Starting,
+
+        /**
+         * Emitted once without _currentDiscoveredFolder when it starts,
+         * then for each folder.
+         */
+        Discovery,
+
+        /// Emitted once when reconcile starts
+        Reconcile,
+
+        /// Emitted during propagation, with progress data
+        Propagation,
+
+        /**
+         * Emitted once when done
+         *
+         * Except when SyncEngine jumps directly to finalize() without going
+         * through slotFinished().
+         */
+        Done
+    };
+
+    Status status() const;
+
     /**
      * Called when propagation starts.
      *
@@ -61,7 +89,7 @@ public:
     /**
      * Increase the file and size totals by the amount indicated in item.
      */
-    void adjustTotalsForFile(const SyncFileItem & item);
+    void adjustTotalsForFile(const SyncFileItem &item);
 
     quint64 totalFiles() const;
     quint64 completedFiles() const;
@@ -73,13 +101,12 @@ public:
     quint64 currentFile() const;
 
     /** Return true if the size needs to be taken in account in the total amount of time */
-    static inline bool isSizeDependent(const SyncFileItem & item)
+    static inline bool isSizeDependent(const SyncFileItem &item)
     {
-        return ! item._isDirectory && (
-               item._instruction == CSYNC_INSTRUCTION_CONFLICT
-            || item._instruction == CSYNC_INSTRUCTION_SYNC
-            || item._instruction == CSYNC_INSTRUCTION_NEW
-            || item._instruction == CSYNC_INSTRUCTION_TYPE_CHANGE);
+        return !item.isDirectory() && (item._instruction == CSYNC_INSTRUCTION_CONFLICT
+                                         || item._instruction == CSYNC_INSTRUCTION_SYNC
+                                         || item._instruction == CSYNC_INSTRUCTION_NEW
+                                         || item._instruction == CSYNC_INSTRUCTION_TYPE_CHANGE);
     }
 
     /**
@@ -142,6 +169,8 @@ public:
         friend class ProgressInfo;
     };
 
+    Status _status;
+
     struct OWNCLOUDSYNC_EXPORT ProgressItem
     {
         SyncFileItem _item;
@@ -152,7 +181,8 @@ public:
     SyncFileItem _lastCompletedItem;
 
     // Used during local and remote update phase
-    QString _currentDiscoveredFolder;
+    QString _currentDiscoveredRemoteFolder;
+    QString _currentDiscoveredLocalFolder;
 
     void setProgressComplete(const SyncFileItem &item);
 
@@ -212,14 +242,23 @@ private:
 
 namespace Progress {
 
-    OWNCLOUDSYNC_EXPORT QString asActionString( const SyncFileItem& item );
-    OWNCLOUDSYNC_EXPORT QString asResultString(  const SyncFileItem& item );
-
-    OWNCLOUDSYNC_EXPORT bool isWarningKind( SyncFileItem::Status );
-    OWNCLOUDSYNC_EXPORT bool isIgnoredKind( SyncFileItem::Status );
+    OWNCLOUDSYNC_EXPORT QString asActionString(const SyncFileItem &item);
+    OWNCLOUDSYNC_EXPORT QString asResultString(const SyncFileItem &item);
 
+    OWNCLOUDSYNC_EXPORT bool isWarningKind(SyncFileItem::Status);
+    OWNCLOUDSYNC_EXPORT bool isIgnoredKind(SyncFileItem::Status);
 }
 
+/** Type of error
+ *
+ * Used for ProgressDispatcher::syncError. May trigger error interactivity
+ * in IssuesWidget.
+ */
+enum class ErrorCategory {
+    Normal,
+    InsufficientRemoteStorage,
+};
+
 /**
  * @file progressdispatcher.h
  * @brief A singleton class to provide sync progress information to other gui classes.
@@ -235,7 +274,7 @@ class OWNCLOUDSYNC_EXPORT ProgressDispatcher : public QObject
 
     friend class Folder; // only allow Folder class to access the setting slots.
 public:
-    static ProgressDispatcher* instance();
+    static ProgressDispatcher *instance();
     ~ProgressDispatcher();
 
 signals:
@@ -246,21 +285,30 @@ signals:
       @param[out]  progress   A struct with all progress info.
 
      */
-    void progressInfo( const QString& folder, const ProgressInfo& progress );
+    void progressInfo(const QString &folder, const ProgressInfo &progress);
     /**
      * @brief: the item was completed by a job
      */
-    void itemCompleted(const QString &folder, const SyncFileItemPtr & item);
+    void itemCompleted(const QString &folder, const SyncFileItemPtr &item);
+
+    /**
+     * @brief A new folder-wide sync error was seen.
+     */
+    void syncError(const QString &folder, const QString &message, ErrorCategory category);
+
+    /**
+     * @brief Emitted for a folder when a sync is done, listing all pending conflicts
+     */
+    void folderConflicts(const QString &folder, const QStringList &conflictPaths);
 
 protected:
-    void setProgressInfo(const QString& folder, const ProgressInfo& progress);
+    void setProgressInfo(const QString &folder, const ProgressInfo &progress);
 
 private:
-    ProgressDispatcher(QObject* parent = 0);
+    ProgressDispatcher(QObject *parent = 0);
 
     QElapsedTimer _timer;
-    static ProgressDispatcher* _instance;
+    static ProgressDispatcher *_instance;
 };
-
 }
 #endif // PROGRESSDISPATCHER_H
